from signal import pause
from time import sleep
from threading import Thread
import sys
import time
from time import strftime
#Import the LCD Libraries
from rpi_lcd import LCD
#Import Luma OLED Libraries
from luma.oled.device import ssd1306, ssd1309, ssd1325, ssd1331, sh1106, ws0010
from luma.core.interface.serial import i2c
from luma.core.render import canvas

#Point LCD, OLEDs to right paths
lcd = LCD()
serial = i2c(port=1, address=0x3C)
device = sh1106(serial, rotate=2)

#Get Crypto Prices
try:
    import requests
except ImportError:
    print("The requests library was not found. Run 'sudo -H pip3 install requests' to install it.")
    sys.exit()

def fetch_price(crypto_currency, fiat_currency):
    bitstamp_api = "https://www.bitstamp.net/api/v2/ticker/" + crypto_currency.lower() + fiat_currency.lower()
    try:
        r = requests.get(bitstamp_api)
        return r.json()
    except:
        print("Error fetching from Bitstamp API")

def price_text(crypto_currency, fiat_currency):
    data = fetch_price(crypto_currency, fiat_currency)
    return [
        '{}/{} {}'.format(crypto_currency, fiat_currency, data['last']),
        '24h Hi {}'.format(data['high']),
        'Lo {}'.format(data['low'])
    ]

#Draw Crypto Prices on OLED
def btc(device):

    with canvas(device) as draw:
        rows = price_text("BTC", "GBP")
        draw.text((0, 10), "PCG Ticker", fill="white")
        draw.text((0, 20), "Bitcoin", fill="white")
        draw.text((0, 30), rows[0], fill="white")
        draw.text((0, 40), rows[1], fill="white")
        draw.text((0, 50), rows[2], fill="white")

def ltc(device):

    with canvas(device) as draw:
        rows = price_text("LTC", "GBP")
        draw.text((0, 10), "PCG Ticker", fill="white")
        draw.text((0, 20), "LiteCoin", fill="white")
        draw.text((0, 30), rows[0], fill="white")
        draw.text((0, 40), rows[1], fill="white")
        draw.text((0, 50), rows[2], fill="white")


def eth(device):

    with canvas(device) as draw:
        rows = price_text("ETH", "GBP")
        draw.text((0, 10), "PCG Ticker", fill="white")
        draw.text((0, 20), "Ethereum", fill="white")
        draw.text((0, 30), rows[0], fill="white")
        draw.text((0, 40), rows[1], fill="white")
        draw.text((0, 50), rows[2], fill="white")

#LCD Time Ticker
def lcd_time():
    while True:
        lcd.text(strftime("%H:%M:%S"),1)
        lcd.text("Time, UK",2)
        time.sleep(.01)

#OLED Crypto Ticker that refreshes to a new currency
def oled_crypto():
    while True:
        btc(device)
        time.sleep(10)
        ltc(device)
        time.sleep(10)
        eth(device)
        time.sleep(10)

#When running from Terminal, update every 30 seconds to show it's working 
def tickerunning():
    while True:
        print("Running Ticker!" + strftime(" %H:%M:%S"))
        time.sleep(30)

#Run threading
try:
    oled_display = Thread(target=oled_crypto, daemon=True)
    lcd_display = Thread(target=lcd_time, daemon=True)
    ticker = Thread(target=tickerunning, daemon=True)
    
    lcd_display.start()
    oled_display.start()
    ticker.start()
    
    pause()

#Keyboard will kill the program
except KeyboardInterrupt:
        pass

#Try and clear the LCD
finally:
    sleep(0.5)
    lcd.clear()
